/**
 * TP JCE
 * 
 * @author : Didier DONSEZ
 */
 
import java.io.*;
import java.security.*;
import java.security.cert.*;
import javax.crypto.*;
import javax.crypto.spec.*;

public class Crypt {

	private static final int BUFSIZ=8096;

    // Cipher the Input Stream
    protected static void transformWithInput(InputStream is, OutputStream os, Cipher c)
	throws Exception {
		CipherInputStream cis = new CipherInputStream( is, c );
		byte[] buffer = new byte[BUFSIZ];
		int len;
		while ((len = cis.read(buffer,0,BUFSIZ)) != -1) {
    	    os.write(buffer,0,len);
		}
    	os.close();
    	cis.close();
    }

    // Cipher the Output stream
    protected static void transformWithOutput(InputStream is, OutputStream os, Cipher c)
	throws Exception {

		CipherOutputStream cos = new CipherOutputStream( os, c );
		byte[] buffer = new byte[BUFSIZ];
		int len;
		while ((len =is.read(buffer,0,BUFSIZ)) != -1) {
    	    cos.write(buffer,0,len);
		}
    	cos.close();
    	is.close();
		/*
		CipherOutputStream cos= new CipherOutputStream( os, c );
		int i;
		while ( (i = is.read()) >= 0 ) {
		    cos.write(i);
		}
		cos.close();
		is.close();
		*/
    }
    

  static Cipher initCipherPBE(int mode, String algoname, String[] args) throws Exception {

    if (args.length != 6 ) { usage(); return null; }
	
    String password=args[5];
	
	// Salt
    byte[] salt = {
                  (byte)0xc7, (byte)0x73, (byte)0x21, (byte)0x8c,
                  (byte)0x7e, (byte)0xc8, (byte)0xee, (byte)0x99
              };
    // Iteration count
    int count = 20;
    // Create PBE parameter set
    PBEParameterSpec pbeParamSpec = new PBEParameterSpec(salt, count);

    // Collect user password as char array and convert
    // it into a SecretKey object, using a PBE key
    // factory.

    PBEKeySpec pbeKeySpec = new PBEKeySpec(password.toCharArray());
    SecretKeyFactory keyFac = SecretKeyFactory.getInstance(algoname);
    SecretKey pbeKey = keyFac.generateSecret(pbeKeySpec);

    // Create PBE Cipher
    Cipher c = Cipher.getInstance(algoname);

    // Initialize PBE Cipher with key and parameters
    c.init(mode, pbeKey, pbeParamSpec);
	
	return c;
  }
  
  static Cipher initCipherRSA(int mode, String algoname, String[] args) throws Exception {
	Key key;
    switch (args.length )  {
    case 9: { // private key from Keystore entry
		String  keystorefile=args[5];
		String  storepass=args[6];
		String  alias=args[7];
		String  keypass=args[8];
	    KeyStore keystore = KeyStore.getInstance("JKS");
    	keystore.load(new FileInputStream(keystorefile), storepass.toCharArray());			
		key=keystore.getKey(alias, keypass.toCharArray());	
		break;
    }
    case 8:{ // public key from KeyStore entry
		String  keystorefile=args[5];
		String  storepass=args[6];
		String  alias=args[7];
	    KeyStore keystore = KeyStore.getInstance("JKS");
    	keystore.load(new FileInputStream(keystorefile), storepass.toCharArray());			
		key=keystore.getCertificate(alias).getPublicKey();
		break;
    }
    case 6 : { // public key from X509 certificate file
		String  certfile=args[5];
		CertificateFactory cf = CertificateFactory.getInstance("X.509");
		InputStream is = new FileInputStream(certfile);
		java.security.cert.Certificate cert = cf.generateCertificate(is);
		key=cert.getPublicKey();
    	break;
    }
	
    default : return null;
	
    }

    // Create  Cipher
    Cipher c = Cipher.getInstance(algoname);

    // Initialize Cipher with key
    c.init(mode, key);
	
	return c;
  }
    
  
  static void usage() throws Exception {
      System.err.println("Usage: Crypt -encrypt filename encryptedfilename -algo algoname algooptions");
      System.err.println("Usage: Crypt -decrypt encryptedfilename filename -algo algoname algooptions");
      System.err.println("Usage: if algoname in PBE*");
      System.err.println("Usage:  then algooptions are : password");
      System.err.println("Usage: if algoname in DES*");
      System.err.println("Usage:  then algooptions are : base64deskey");
      System.err.println("Usage: if algoname in RSA*");
      System.err.println("Usage:  then algooptions gets the private key: keystore storepass alias entrypass");
      System.err.println("Usage:  or   algooptions gets the public key : keystore storepass alias");
      System.err.println("Usage:  or   algooptions gets the public key : x509certfile");
      return;
  }

  public static void main(String[] args) throws Exception {

    if (args.length < 6 ) { usage(); return; }

    String pmode = args[0];
	int mode;
	if(pmode.equals("-encrypt")) {
		mode=Cipher.ENCRYPT_MODE;
	} else if (pmode.equals("-decrypt")){ 
		mode=Cipher.DECRYPT_MODE;
	} else {
		usage(); return;
	}
	
    String filein = args[1];
    String fileout = args[2];
	String algoname = args[4];

	Cipher c;

	if (algoname.startsWith("PBE"))	{
		// PBEWithMD5AndDES (PKCS#5) in ABA and J2SE 1.4
    	// PBEWithSHA1And128BitRC4 (PKCS#12) in ABA
		c=initCipherPBE(mode, algoname, args);
	} else	if (algoname.startsWith("RSA")) {
		// RSA/ECB/PKCS1Padding
		c=initCipherRSA(mode, algoname, args);
	} else {
		System.err.println("Crypt: unknown algoname \""+algoname+"\"");
        return;
	}	
	
	if (c==null) { usage(); return; }
	
	// Open files
	InputStream in;
    if(!filein.equals("-")){
		in = new FileInputStream(filein);
    } else {
		in = System.in;
    }
	
	OutputStream out;
	if(!fileout.equals("-")){
    	out = new FileOutputStream(fileout);
	} else {
		out = System.out;
	}
	
    // Copy and transform input file in output file
    transformWithInput(in, out, c);
  }
  
}
